@@ -7,45 +7,54 @@ use Google::Adwords::AdGroup;
 
 sub test_class { return "Google::Adwords::AdGroupService"; }
 
+# tests to be run
+my %tests = (
+    addAdGroup              => 1,
+    addAdGroupList          => 1,
+    getAdGroup              => 1,
+    getAdGroupList          => 1,
+    getAdGroupStats         => 1, 
+    getAllAdGroups          => 1,
+    updateAdGroup           => 1,
+    updateAdGroupList       => 1,
+);
+
 sub addAdGroup : Test(no_plan)
 {
     my $self = shift;
 
-    #return;
+    $sub_name = (caller 0)[3];
+    $sub_name =~ s/^.+:://;
+    if (not $tests{$sub_name}) {
+        return;
+    }
+
 
     if ($self->{sandbox}) {
-        $self->{obj}->debug(1);
         
         my $adgroup = Google::Adwords::AdGroup->new();
         $adgroup->maxCpc(100000);
-        my $campaign_id = 4937;
-
+        my $campaign_id = $self->_get_campaign_id();
+        if (not defined $campaign_id) {
+            diag("addCampaign was not called");
+            return;
+        }
         my $adgroup_response = $self->{obj}->addAdGroup($campaign_id, $adgroup);
-        ok ($adgroup_response->campaignId == 4937, 'addAdGroup');
+        ok ($adgroup_response->campaignId == $campaign_id, 'addAdGroup id: ' 
+            . $adgroup_response->id);
+
+        # save for further use
+        $self->{_adgroup_id} = $adgroup_response->id;
+
+        # save for further module tests
+        $self->_set_adgroup_id($adgroup_response->id);
+
     }
     else {
 
         my $soap = Test::MockModule->new('SOAP::Lite');
         $soap->mock( call => sub {
             my $xml .= <<'EOF';
-<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/"
-xmlns:xsd="http://www.w3.org/2001/XMLSchema"
-xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
- <soapenv:Header>
-  <responseTime soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">113</responseTime>
-  <operations soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">1</operations>
-  <units soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">1</units>
-  <requestId soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">54d9ad57aa6d1fa29a253bf66b4717ce</requestId>
- </soapenv:Header>
- <soapenv:Body>
   <addAdGroupResponse xmlns="">
    <ns1:addAdGroupReturn
 xmlns:ns1="https://adwords.google.com/api/adwords/v5">
@@ -56,10 +65,9 @@ xmlns:ns1="https://adwords.google.com/api/adwords/v5">
     <ns1:status>Enabled</ns1:status>
    </ns1:addAdGroupReturn>
   </addAdGroupResponse>
- </soapenv:Body>
-</soapenv:Envelope>
 EOF
 
+            $xml = $self->gen_full_response($xml);
             my $env = SOAP::Deserializer->deserialize($xml);
             return $env;
         });
@@ -72,11 +80,11 @@ EOF
         ok ($adgroup_response->campaignId == 4937, 'addAdGroup');
         
         # check that we got the response headers
-        ok ($self->{obj}->requestId eq '54d9ad57aa6d1fa29a253bf66b4717ce',
-                'Response header');
-        ok ($self->{obj}->responseTime == 113, 'Response header');
-        ok ($self->{obj}->operations == 1, 'Response header');
-        ok ($self->{obj}->units == 1, 'Response header');
+        ok ($self->{obj}->requestId eq 'f7912565442e4adeb1bf30cbbf2f8fd2',
+                'Response header (requestId)');
+        ok ($self->{obj}->responseTime == 39, 'Response header (responseTime)');
+        ok ($self->{obj}->operations == 1, 'Response header (operations)');
+        ok ($self->{obj}->units == 1, 'Response header (units)');
 
 
     }
@@ -87,13 +95,22 @@ sub addAdGroupList : Test(no_plan)
 {
     my $self = shift;
 
-    #return;
+
+    $sub_name = (caller 0)[3];
+    $sub_name =~ s/^.+:://;
+    if (not $tests{$sub_name}) {
+        return;
+    }
 
     if ($self->{sandbox}) {
         
-        $self->{obj}->debug(1);
 
-        my $campaign_id = 1004;
+        my $campaign_id = $self->_get_campaign_id();
+        if (not defined $campaign_id) {
+            diag("addCampaign was not called");
+            return;
+        }
+
         my @adgroups_to_add;
 
         my $adgroup1 = Google::Adwords::AdGroup->new;
@@ -107,32 +124,22 @@ sub addAdGroupList : Test(no_plan)
             $campaign_id, \@adgroups_to_add
         );
 
-        ok ($new_adgroups[0]->campaignId == 4937, 'addAdGroupList');
-        ok ($new_adgroups[1]->campaignId == 4937, 'addAdGroupList');
+        ok ($new_adgroups[0]->campaignId == $campaign_id, 'addAdGroupList');
+        ok ($new_adgroups[1]->campaignId == $campaign_id, 'addAdGroupList');
+        ok ($new_adgroups[0]->id =~ /\d+/, 
+            'addAdGroupList id: ' . $new_adgroups[0]->id);
+        ok ($new_adgroups[1]->id =~ /\d+/, 
+            'addAdGroupList id: ' . $new_adgroups[1]->id);
+
+        # save for further use
+        $self->{_adgroup_id_0} = $new_adgroups[0]->id;
+        $self->{_adgroup_id_1} = $new_adgroups[1]->id;
     }
     else {
 
         my $soap = Test::MockModule->new('SOAP::Lite');
         $soap->mock( call => sub {
             my $xml .= <<'EOF';
-<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/"
-xmlns:xsd="http://www.w3.org/2001/XMLSchema"
-xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
- <soapenv:Header>
-  <responseTime soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">113</responseTime>
-  <operations soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">1</operations>
-  <units soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">1</units>
-  <requestId soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">54d9ad57aa6d1fa29a253bf66b4717ce</requestId>
- </soapenv:Header>
- <soapenv:Body>
    <addAdGroupListResponse xmlns="">
    <ns1:addAdGroupListReturn
 xmlns:ns1="https://adwords.google.com/api/adwords/v5">
@@ -151,10 +158,9 @@ xmlns:ns2="https://adwords.google.com/api/adwords/v5">
     <ns2:status>Enabled</ns2:status>
    </ns2:addAdGroupListReturn>
   </addAdGroupListResponse>
- </soapenv:Body>
-</soapenv:Envelope>
 EOF
 
+            $xml = $self->gen_full_response($xml);
             my $env = SOAP::Deserializer->deserialize($xml);
             return $env;
         });
@@ -187,39 +193,25 @@ sub getAdGroup : Test(no_plan)
 {
     my $self = shift;
 
-    #return;
+    $sub_name = (caller 0)[3];
+    $sub_name =~ s/^.+:://;
+    if (not $tests{$sub_name}) {
+        return;
+    }
+
 
     if ($self->{sandbox}) {
-        $self->{obj}->debug(1);
         
-        my $adgroup = $self->{obj}->getAdGroup(200531);
+        my $adgroup = $self->{obj}->getAdGroup($self->{_adgroup_id});
 
-        ok ($adgroup->campaignId == 4937, 'getAdGroup');
-        ok ($adgroup->id == 200531, 'getAdGroup');
+        ok ($adgroup->campaignId == $self->_get_campaign_id(), 'getAdGroup');
+        ok ($adgroup->id == $self->{_adgroup_id}, 'getAdGroup');
     }
     else {
 
         my $soap = Test::MockModule->new('SOAP::Lite');
         $soap->mock( call => sub {
             my $xml .= <<'EOF';
-<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/"
-xmlns:xsd="http://www.w3.org/2001/XMLSchema"
-xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
- <soapenv:Header>
-  <responseTime soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">113</responseTime>
-  <operations soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">1</operations>
-  <units soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">1</units>
-  <requestId soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">54d9ad57aa6d1fa29a253bf66b4717ce</requestId>
- </soapenv:Header>
- <soapenv:Body>
   <getAdGroupResponse xmlns="">
    <ns1:getAdGroupReturn
 xmlns:ns1="https://adwords.google.com/api/adwords/v5">
@@ -230,10 +222,9 @@ xmlns:ns1="https://adwords.google.com/api/adwords/v5">
     <ns1:status>Enabled</ns1:status>
    </ns1:getAdGroupReturn>
   </getAdGroupResponse>
- </soapenv:Body>
-</soapenv:Envelope>
 EOF
 
+            $xml = $self->gen_full_response($xml);
             my $env = SOAP::Deserializer->deserialize($xml);
             return $env;
         });
@@ -252,39 +243,29 @@ sub getAdGroupList : Test(no_plan)
 {
     my $self = shift;
 
-    #return;
+    $sub_name = (caller 0)[3];
+    $sub_name =~ s/^.+:://;
+    if (not $tests{$sub_name}) {
+        return;
+    }
+
 
     if ($self->{sandbox}) {
-        $self->{obj}->debug(1);
         
-        my @adgroups = $self->{obj}->getAdGroupList(qw/200531 200532/);
+        my @adgroup_ids = (
+            $self->{_adgroup_id_0},
+            $self->{_adgroup_id_1},
+        );
+        my @adgroups = $self->{obj}->getAdGroupList(@adgroup_ids);
         
-        ok ($adgroups[0]->id == 200531, 'getAdGroupList');
-        ok ($adgroups[1]->id == 200532, 'getAdGroupList');
+        ok ($adgroups[0]->id == $self->{_adgroup_id_0}, 'getAdGroupList');
+        ok ($adgroups[1]->id == $self->{_adgroup_id_1}, 'getAdGroupList');
     }
     else {
 
         my $soap = Test::MockModule->new('SOAP::Lite');
         $soap->mock( call => sub {
             my $xml .= <<'EOF';
-<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/"
-xmlns:xsd="http://www.w3.org/2001/XMLSchema"
-xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
- <soapenv:Header>
-  <responseTime soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">113</responseTime>
-  <operations soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">1</operations>
-  <units soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">1</units>
-  <requestId soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">54d9ad57aa6d1fa29a253bf66b4717ce</requestId>
- </soapenv:Header>
- <soapenv:Body>
   <getAdGroupListResponse xmlns="">
    <ns1:getAdGroupListReturn
 xmlns:ns1="https://adwords.google.com/api/adwords/v5">
@@ -303,10 +284,9 @@ xmlns:ns1="https://adwords.google.com/api/adwords/v5">
     <ns1:status>Enabled</ns1:status>
    </ns1:getAdGroupListReturn>
   </getAdGroupListResponse>
- </soapenv:Body>
-</soapenv:Envelope>
 EOF
 
+            $xml = $self->gen_full_response($xml);
             my $env = SOAP::Deserializer->deserialize($xml);
             return $env;
         });
@@ -321,43 +301,33 @@ EOF
 
 }
 
-sub b_getAllAdGroups : Test(no_plan)
+sub getAllAdGroups : Test(no_plan)
 {
     my $self = shift;
 
-    #return;
+    $sub_name = (caller 0)[3];
+    $sub_name =~ s/^.+:://;
+    if (not $tests{$sub_name}) {
+        return;
+    }
+
 
     if ($self->{sandbox}) {
-        $self->{obj}->debug(1);
         
-        my @adgroups = $self->{obj}->getAllAdGroups(1004);
+        my @adgroups = $self->{obj}->getAllAdGroups($self->_get_campaign_id);
         
-        ok ($adgroups[0]->campaignId == 4937, 'getAllAdGroups');
-
+        # must get three or more
+        ok (scalar @adgroups >= 3, 'getAllAdGroups');
+        for (@adgroups) {
+            ok ($_->id =~ /\d+/, 
+                'getAllAdGroups id: ' . $_->id);
+        }
     }
     else {
 
         my $soap = Test::MockModule->new('SOAP::Lite');
         $soap->mock( call => sub {
             my $xml .= <<'EOF';
-<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/"
-xmlns:xsd="http://www.w3.org/2001/XMLSchema"
-xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
- <soapenv:Header>
-  <responseTime soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">113</responseTime>
-  <operations soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">1</operations>
-  <units soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">1</units>
-  <requestId soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">54d9ad57aa6d1fa29a253bf66b4717ce</requestId>
- </soapenv:Header>
- <soapenv:Body>
   <getAllAdGroupsResponse xmlns="">
    <ns1:getAllAdGroupsReturn
 xmlns:ns1="https://adwords.google.com/api/adwords/v5">
@@ -376,10 +346,9 @@ xmlns:ns1="https://adwords.google.com/api/adwords/v5">
     <ns1:status>Enabled</ns1:status>
    </ns1:getAllAdGroupsReturn>
   </getAllAdGroupsResponse>
- </soapenv:Body>
-</soapenv:Envelope>
 EOF
 
+            $xml = $self->gen_full_response($xml);
             my $env = SOAP::Deserializer->deserialize($xml);
             return $env;
         });
@@ -397,24 +366,32 @@ sub getAdGroupStats : Test(no_plan)
 {
     my $self = shift;
 
-    #return;
+    $sub_name = (caller 0)[3];
+    $sub_name =~ s/^.+:://;
+    if (not $tests{$sub_name}) {
+        return;
+    }
+
 
     if ($self->{sandbox}) {
 
-        $self->{obj}->debug(1);
 
-        my $campaign_id = 1004;
+        my $campaign_id = $self->_get_campaign_id;
+        if (not defined $campaign_id) {
+            diag("addCampaign was not called");
+            return;
+        }
 
         my @stats = $self->{obj}->getAdGroupStats({
             campaignId => $campaign_id,
-            adGroupIds => [ 1001, 1002 ],
-            startDay => '2006-09-01',
-            endDay => '2006-09-15',
+            adGroupIds => [ $self->{_adgroup_id_0}, $self->{_adgroup_id_1}, ],
+            startDay => '2006-12-01',
+            endDay => '2006-12-15',
             inPST => 1,
         });
 
-        ok ($stats[0]->id == 1001, 'getAdGroupStats');
-        ok ($stats[1]->id == 1002, 'getAdGroupStats');
+        ok ($stats[0]->id == $self->{_adgroup_id_0}, 'getAdGroupStats');
+        ok ($stats[1]->id == $self->{_adgroup_id_1}, 'getAdGroupStats');
 
     }
     else {
@@ -422,24 +399,6 @@ sub getAdGroupStats : Test(no_plan)
         my $soap = Test::MockModule->new('SOAP::Lite');
         $soap->mock( call => sub {
             my $xml .= <<'EOF';
-<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/"
-xmlns:xsd="http://www.w3.org/2001/XMLSchema"
-xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
- <soapenv:Header>
-  <responseTime soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">113</responseTime>
-  <operations soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">1</operations>
-  <units soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">1</units>
-  <requestId soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">54d9ad57aa6d1fa29a253bf66b4717ce</requestId>
- </soapenv:Header>
- <soapenv:Body>
 <getAdGroupStatsResponse xmlns="">
    <ns1:getAdGroupStatsReturn
 xmlns:ns1="https://adwords.google.com/api/adwords/v5">
@@ -462,10 +421,9 @@ xmlns:ns2="https://adwords.google.com/api/adwords/v5">
     <ns2:impressions>0</ns2:impressions>
    </ns2:getAdGroupStatsReturn>
   </getAdGroupStatsResponse>
- </soapenv:Body>
-</soapenv:Envelope>
 EOF
 
+            $xml = $self->gen_full_response($xml);
             my $env = SOAP::Deserializer->deserialize($xml);
             return $env;
         });
@@ -495,14 +453,18 @@ sub updateAdGroup : Test(no_plan)
 {
     my $self = shift;
 
-    #return;
+    $sub_name = (caller 0)[3];
+    $sub_name =~ s/^.+:://;
+    if (not $tests{$sub_name}) {
+        return;
+    }
+
 
     if ($self->{sandbox}) {
 
-        $self->{obj}->debug(1);
 
         my $adgroup = Google::Adwords::AdGroup->new;
-        $adgroup->id(1001);
+        $adgroup->id($self->{_adgroup_id});
         $adgroup->maxCpc(5000000);
 
         my $ret = $self->{obj}->updateAdGroup($adgroup);
@@ -516,29 +478,10 @@ sub updateAdGroup : Test(no_plan)
         my $soap = Test::MockModule->new('SOAP::Lite');
         $soap->mock( call => sub {
             my $xml .= <<'EOF';
-<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/"
-xmlns:xsd="http://www.w3.org/2001/XMLSchema"
-xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
- <soapenv:Header>
-  <responseTime soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">113</responseTime>
-  <operations soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">1</operations>
-  <units soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">1</units>
-  <requestId soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">54d9ad57aa6d1fa29a253bf66b4717ce</requestId>
- </soapenv:Header>
- <soapenv:Body>
     <updateAdGroupResponse xmlns=""/>
- </soapenv:Body>
-</soapenv:Envelope>
 EOF
 
+            $xml = $self->gen_full_response($xml);
             my $env = SOAP::Deserializer->deserialize($xml);
             return $env;
         });
@@ -559,17 +502,21 @@ sub updateAdGroupList : Test(no_plan)
 {
     my $self = shift;
 
-    #return;
+    $sub_name = (caller 0)[3];
+    $sub_name =~ s/^.+:://;
+    if (not $tests{$sub_name}) {
+        return;
+    }
+
 
     if ($self->{sandbox}) {
 
-        $self->{obj}->debug(1);
 
         my $adgroup1 = Google::Adwords::AdGroup->new;
-        $adgroup1->id(1001);
+        $adgroup1->id($self->{_adgroup_id_0});
         $adgroup1->maxCpc(5000000);
         my $adgroup2 = Google::Adwords::AdGroup->new;
-        $adgroup2->id(1002);
+        $adgroup2->id($self->{_adgroup_id_1});
         $adgroup2->maxCpc(5000000);
 
         my @adgroups;
@@ -586,29 +533,10 @@ sub updateAdGroupList : Test(no_plan)
         my $soap = Test::MockModule->new('SOAP::Lite');
         $soap->mock( call => sub {
             my $xml .= <<'EOF';
-<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/"
-xmlns:xsd="http://www.w3.org/2001/XMLSchema"
-xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
- <soapenv:Header>
-  <responseTime soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">113</responseTime>
-  <operations soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">1</operations>
-  <units soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">1</units>
-  <requestId soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next"
-soapenv:mustUnderstand="0"
-xmlns="https://adwords.google.com/api/adwords/v5">54d9ad57aa6d1fa29a253bf66b4717ce</requestId>
- </soapenv:Header>
- <soapenv:Body>
     <updateAdGroupListResponse xmlns=""/>
- </soapenv:Body>
-</soapenv:Envelope>
 EOF
 
+            $xml = $self->gen_full_response($xml);
             my $env = SOAP::Deserializer->deserialize($xml);
             return $env;
         });
